@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 1999 Balzs Scheidler
- * Copyright (c) 1999 BalaBit Computing
+ * Copyright (c) 1999-2001 BalaBit IT Ltd.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: affile.c,v 1.33 2001/01/19 09:31:05 bazsi Exp $
+ * $Id: affile.c,v 1.34 2001/02/24 12:38:02 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -47,7 +47,7 @@
 #include "affile.c.x"
 
 struct ol_string *
-expand_macros(struct syslog_config *cfg, struct ol_string *template, struct log_info *msg);
+expand_macros(struct syslog_config *cfg, struct ol_string *template, int template_escape, struct log_info *msg);
 
 
 static int do_open_file(struct ol_string *name, int flags, 
@@ -219,8 +219,11 @@ do_init_dest_writer(struct log_handler *c, struct syslog_config *cfg, struct per
 				 NULL);
 		self->res = REMEMBER_RESOURCE(cfg->resources, &self->dest->super.super);
 		self->time_reap = cfg->time_reap;
-		if ((self->owner->flags & AFFILE_NO_EXPAND) == 0)
+		if ((self->owner->flags & AFFILE_NO_EXPAND) == 0) {
 			self->reap = io_callout(cfg->backend, cfg->time_reap, make_affile_dest_reaper(cfg, self));
+			/* we want to be called on reinitialization */
+			io_callout_set_drop(self->reap, 0);
+		}
 		/* self->attempts = 0; */
 	}
 	else {
@@ -238,7 +241,7 @@ static void do_handle_dest_writer(struct log_handler *c, struct log_info *msg)
 	if (self->dest) {
 		struct ol_string *l;
 		if (self->owner->template_output)
-			l = expand_macros(self->owner->cfg, self->owner->template_output, msg);
+			l = expand_macros(self->owner->cfg, self->owner->template_output, self->owner->template_escape, msg);
 		else
 			l = c_format("%S %S %S\n", msg->date, msg->host, msg->msg);
 		A_WRITE_STRING(&self->dest->buffer->super, l);
@@ -274,6 +277,7 @@ struct affile_dest_writer *make_affile_dest_writer(struct affile_dest *owner, st
 	 (sync_freq simple UINT32)
 	 (template_fname string)
 	 (template_output string)
+	 (template_escape . int)
 	 (writers object affile_dest_writer)
 	 (cfg object syslog_config)))
 */
@@ -378,6 +382,13 @@ void affile_set_file_template(struct log_dest_driver *c, char *t)
 	self->template_output = c_format("%z", t);
 }
 
+void affile_set_template_escape(struct log_dest_driver *c, int enable)
+{
+	CAST(affile_dest, self, c);
+
+	self->template_escape = enable;
+}
+
 #define MAX_MACRO_ARGS        32
 #define MAX_EXPANDED_MACRO    2048
 
@@ -406,7 +417,35 @@ struct macro_def {
 
 #define M_MESSAGE  15
 
-void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, struct log_info *msg)
+int append_string(char **dest, int *left, char *str, int length, int escape)
+{
+	int l;
+  
+	if (!escape) {
+		l = MIN(length, *left - 1);
+		strncpy(*dest, str, l);
+	}
+	else {
+		char *p;
+      
+		l = 0;
+		for (p = str; length && ((*left - l) > 1); p++, length--, l++) {
+			if (*p == '\'' || *p == '\"' || *p == '\\') {
+				if ((*left - l) < 3)
+					break;
+				*(*dest + l) = '\\';
+				*(*dest + l + 1) = *p;
+				l++;
+			}
+			else {
+				*(*dest + l) = *p;
+			}
+		}
+	}
+	return l;
+}
+
+void expand_macro(struct syslog_config *cfg, int id, int escape, char **dest, int *left, struct log_info *msg)
 {
 	int length = 0;
 	
@@ -415,8 +454,7 @@ void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, str
 		/* facility */
 		char *n = syslog_lookup_value(msg->pri & LOG_FACMASK, sl_facilities);
 		if (n) {
-			length = MIN(*left, strlen(n));
-			memcpy(*dest, n, length);
+			length = append_string(dest, left, n, strlen(n), 0);
 		}
 		else {
 			length = snprintf(*dest, *left, "%x", (msg->pri & LOG_FACMASK) >> 3);
@@ -427,8 +465,7 @@ void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, str
 		/* level */
 		char *n = syslog_lookup_value(msg->pri & LOG_PRIMASK, sl_levels);
 		if (n) {
-			length = MIN(*left, strlen(n));
-			memcpy(*dest, n, length);
+			length = append_string(dest, left, n, strlen(n), 0);
 		}
 		else {
 			/* should never happen */
@@ -443,7 +480,7 @@ void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, str
 	}
 	case M_FULLHOST: {
 		/* full hostname */
-		length = snprintf(*dest, *left, "%s", msg->host->data);
+		length = append_string(dest, left, msg->host->data, msg->host->length, escape);
 		break;
 	}
 	case M_HOST: {
@@ -464,15 +501,13 @@ void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, str
 		else {
 			length = MIN(*left, msg->host->length - (p1 - msg->host->data));
 		}
-		memcpy(*dest, p1, length);
-
+		length = append_string(dest, left, p1, length, escape);
 		break;
 	}
 	case M_PROGRAM: {
 		/* program */
 		if (msg->program) {
-			length = MIN(*left, msg->program->length);
-			memcpy(*dest, msg->program->data, length);
+			length = append_string(dest, left, msg->program->data, msg->program->length, escape);
 		}
 		break;
 	}
@@ -537,7 +572,7 @@ void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, str
 		break;
 	case M_MESSAGE: {
 		/* message */
-		length = snprintf(*dest, *left, "%s", msg->msg->data);
+		length = append_string(dest, left, msg->msg->data, msg->msg->length, escape);
 		break;
 	}
 	}
@@ -549,7 +584,7 @@ void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, str
 }
 
 struct ol_string *
-expand_macros(struct syslog_config *cfg, struct ol_string *template, struct log_info *msg)
+expand_macros(struct syslog_config *cfg, struct ol_string *template, int template_escape, struct log_info *msg)
 {
 	static struct macro_def macros[] = {
 		{ "FACILITY", M_FACILITY },
@@ -597,7 +632,7 @@ expand_macros(struct syslog_config *cfg, struct ol_string *template, struct log_
 			}
 			else {
 				i += strlen(macros[j].name) + 1;
-				expand_macro(cfg, macros[j].id, &format_ptr, &left, msg);
+				expand_macro(cfg, macros[j].id, template_escape, &format_ptr, &left, msg);
 			}
 		}
 		else {
@@ -620,15 +655,16 @@ affile_find_writer(struct affile_dest *self,
 	UINT32 res;	
 
 	if (self->flags & AFFILE_NO_EXPAND) {
-		if (self->writers) 
-			return self->writers;
-		self->writers = make_affile_dest_writer(self, self->template_fname);
-		res = LOG_HANDLER_INIT(self->writers, self->cfg, NULL);
-		if (res & ST_QUIT)
-			return NULL;
+		if (!self->writers) {
+			self->writers = make_affile_dest_writer(self, self->template_fname);
+			res = LOG_HANDLER_INIT(self->writers, self->cfg, NULL);
+			if (res & ST_QUIT)
+				return NULL;
+		}
+		return self->writers;
 	}
 
-	expanded_fname = expand_macros(self->cfg, self->template_fname, msg);
+	expanded_fname = expand_macros(self->cfg, self->template_fname, 0, msg);
 	p = self->writers;
 	while (p && 
 	       ((p->expanded_fname->length != expanded_fname->length) || 
@@ -709,6 +745,7 @@ struct log_dest_driver *make_affile_dest(const char *name, int flags)
 	self->dir_uid = self->dir_gid = -1;
 	self->dir_perm = -1;
 	self->template_fname = c_format_cstring("%z", name);
+	self->template_escape = 1;
 	if (strchr(name, '$') == NULL)
 		flags |= AFFILE_NO_EXPAND;
 	self->flags = flags;
