@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: afprogram.c,v 1.3.2.2 2001/07/11 14:08:16 bazsi Exp $
+ * $Id: afprogram.c,v 1.4 2001/02/24 12:38:02 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -100,7 +100,7 @@ do_init_afprogram_dest(struct log_handler *c,
 		REMEMBER_RESOURCE(cfg->resources, &prg->super);
 
 		close(msgpipe[0]);
-		self->dest = io_write(make_io_fd(cfg->backend, msgpipe[1], NULL),
+		self->dest = io_write(make_io_fd(cfg->backend, msgpipe[1]),
 				      make_pkt_buffer(cfg->log_fifo_size),
 				      NULL);
 		REMEMBER_RESOURCE(cfg->resources, &self->dest->super.super);
