@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: afstreams.c,v 1.10.2.2 2001/10/19 12:44:59 bazsi Exp $
+ * $Id: afstreams.c,v 1.11 2001/02/24 12:38:02 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -164,7 +164,7 @@ struct nonblocking_fd *io_stream_get(struct io_backend *backend,
 {
 	NEW(stream_fd, f);
 
-	init_file(backend, &f->super, fd, NULL);
+	init_file(backend, &f->super, fd);
 	f->super.prepare = prepare_stream_fd;
 	f->super.read = stream_read_callback;
 	f->super.want_read = 1;
