@@ -37,36 +37,41 @@ typedef union {
 #define	KW_DIR_GROUP	287
 #define	KW_DIR_PERM	288
 #define	KW_FILE_TEMPLATE	289
-#define	KW_OWNER	290
-#define	KW_GROUP	291
-#define	KW_PERM	292
-#define	KW_KEEP_ALIVE	293
-#define	KW_MAX_CONNECTIONS	294
-#define	KW_LOCALIP	295
-#define	KW_IP	296
-#define	KW_LOCALPORT	297
-#define	KW_PORT	298
-#define	KW_DESTPORT	299
-#define	KW_COMPRESS	300
-#define	KW_MAC	301
-#define	KW_AUTH	302
-#define	KW_ENCRYPT	303
-#define	KW_FACILITY	304
-#define	KW_LEVEL	305
-#define	KW_HOST	306
-#define	KW_MATCH	307
-#define	KW_YES	308
-#define	KW_NO	309
-#define	KW_REQUIRED	310
-#define	KW_ALLOW	311
-#define	KW_DENY	312
-#define	DOTDOT	313
-#define	IDENTIFIER	314
-#define	NUMBER	315
-#define	STRING	316
-#define	KW_OR	317
-#define	KW_AND	318
-#define	KW_NOT	319
+#define	KW_TEMPLATE_ESCAPE	290
+#define	KW_OWNER	291
+#define	KW_GROUP	292
+#define	KW_PERM	293
+#define	KW_KEEP_ALIVE	294
+#define	KW_MAX_CONNECTIONS	295
+#define	KW_LOCALIP	296
+#define	KW_IP	297
+#define	KW_LOCALPORT	298
+#define	KW_PORT	299
+#define	KW_DESTPORT	300
+#define	KW_COMPRESS	301
+#define	KW_MAC	302
+#define	KW_AUTH	303
+#define	KW_ENCRYPT	304
+#define	KW_DNS_CACHE	305
+#define	KW_DNS_CACHE_SIZE	306
+#define	KW_DNS_CACHE_EXPIRE	307
+#define	KW_DNS_CACHE_EXPIRE_FAILED	308
+#define	KW_FACILITY	309
+#define	KW_LEVEL	310
+#define	KW_HOST	311
+#define	KW_MATCH	312
+#define	KW_YES	313
+#define	KW_NO	314
+#define	KW_REQUIRED	315
+#define	KW_ALLOW	316
+#define	KW_DENY	317
+#define	DOTDOT	318
+#define	IDENTIFIER	319
+#define	NUMBER	320
+#define	STRING	321
+#define	KW_OR	322
+#define	KW_AND	323
+#define	KW_NOT	324
 
 
 extern YYSTYPE yylval;
