@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 1999 Balzs Scheidler
- * Copyright (c) 1999 BalaBit Computing
+ * Copyright (c) 1999-2001 BalaBit IT Ltd.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: cfgfile.h,v 1.19 2001/01/15 09:45:44 bazsi Exp $
+ * $Id: cfgfile.h,v 1.20 2001/02/24 12:38:03 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -62,6 +62,11 @@
 	(dir_uid simple int)
 	(dir_gid simple int)
 	(dir_perm simple int)
+	(dns_cache simple UINT32)
+	(dns_cache_size simple UINT32)
+	(dns_cache_expire simple UINT32)
+	(dns_cache_expire_failed simple UINT32)
+	(cache simple "struct nscache *")
 	(gc_busy_threshold simple UINT32)
 	(gc_idle_threshold simple UINT32)
 	; internal messages should come from this source
