@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 1999 Balzs Scheidler
- * Copyright (c) 1999 BalaBit Computing
+ * Copyright (c) 1999-2001 BalaBit IT Ltd.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: main.c,v 1.28 2000/11/29 11:21:31 bazsi Exp $
+ * $Id: main.c,v 1.29 2001/02/24 12:38:03 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -44,8 +44,6 @@
 char cfgfilename[128] = PATH_SYSLOG_NG_CONF;
 char pidfilename[128] = PATH_PIDFILE;
 
-int restarting = 0;
-
 /* CLASS:
    (class
      (name reread_config)
@@ -76,7 +74,7 @@ static void do_reread_config(struct callback *c)
 			self->backend->oldconfig = NULL;
 			kill_persistent_config(self->backend->persistent);
 			self->backend->persistent = NULL;
-			restarting = 0;
+			self->backend->super.reloading = 0;
 		}
 		else {
 			/* initializing the new configuration was not 
@@ -93,11 +91,11 @@ static void do_reread_config(struct callback *c)
 		}
 		else {
 			/* this is a fatal error, but no stderr & no log output */
-			exit(137);
+			exit(127);
 		}
 		kill_persistent_config(self->backend->persistent);
 		self->backend->persistent = NULL;
-		restarting = 0;
+		self->backend->super.reloading = 0;
 		break;
 	}
 }
@@ -148,27 +146,24 @@ int main_loop(struct syslog_backend *backend)
 	while (io_iter(&backend->super)) {
 		if (exit_main_loop)
 			break;
-		if (sighuprecvd && !restarting) {
-
-		       	FOR_QUEUE(&backend->super.callouts, struct ol_queue_node *, n) {
-        			ol_queue_remove(n);
-       				ol_space_free(n);
-	        	}
+		if (sighuprecvd && !backend->super.reloading) {
 
 			notice("SIGHUP received, restarting syslog-ng\n");
-			restarting = 1;
+			backend->super.reloading = 1;
 
 			backend->newconfig = make_syslog_config(cfgfilename, &backend->super);
 			if (backend->newconfig) {
 				static struct reread_config r = 
 				{ { STATIC_HEADER, do_reread_config }, NULL, 0 };
 				
+				io_callout_flush(&backend->super);
+				
 				r.backend = backend;
 				r.state = 0;
 				io_callout(&backend->super, 0, &r.super);
 			}
 			else {
-				restarting = 0;
+				backend->super.reloading = 0;
 				notice("Syntax error reading config file, using old configuration\n");
 			}
 			sighuprecvd = 0;
