@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 1999 Balzs Scheidler
- * Copyright (c) 1999 BalaBit Computing
+ * Copyright (c) 1999-2001 BalaBit IT Ltd.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: sources.c,v 1.26 2000/11/29 11:21:31 bazsi Exp $
+ * $Id: sources.c,v 1.27 2001/02/24 12:38:03 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -40,7 +40,7 @@
 #undef CLASS_DEFINE
 
 #include "sources.c.x"
-
+#include "nscache.h"
 /* CLASS:
      (class
        (name log_reader)
@@ -148,22 +148,32 @@ make_log_reader(UINT32 dgram,
 
 /* source_group */
 
-static struct ol_string *get_source_hostname(struct address_info *a, int usedns, int usefqdn)
+static struct ol_string *get_source_hostname(struct address_info *a, int usedns, int usefqdn, struct nscache *cache)
 {
 	struct ol_string *name;
 	static struct ol_string *hostname = NULL;
 	
 	if (a && a->super.isa == &inet_address_info_class) {
 		CAST(inet_address_info, inet_addr, a);
-	        struct hostent *hp;
 		char *hname, *p;
 
-		hp = usedns ? gethostbyaddr((char *) &(inet_addr->sa.sin_addr), sizeof(struct in_addr), AF_INET) : NULL;
-		if (!hp) {
+		if (usedns && cache) {
+			hname = nscache_lookup(cache, inet_addr->sa.sin_addr);
+		}
+		else if (usedns) {
+			struct hostent *hp;
+
+			hp = gethostbyaddr((char *) &(inet_addr->sa.sin_addr),
+					   sizeof(struct in_addr), AF_INET);
+			hname = (hp && hp->h_name) ? hp->h_name : NULL;
+		} 
+		else
+			hname = NULL;
+
+		if (!hname) {
 			return ol_string_use(inet_addr->ip);
 		}
 		else {
-			hname = hp->h_name;
 			if (!usefqdn) {
 				p = strchr(hname, '.');
 				if (p) *p = 0;
@@ -196,7 +206,8 @@ static void do_add_source_name(struct log_handler *c, struct log_info *logmsg)
 	logmsg->source = c;
 	if (!self->keep_hostname || !logmsg->host) {
 		name = get_source_hostname(logmsg->saddr, 
-					   self->use_dns, self->use_fqdn);
+					   self->use_dns, self->use_fqdn,
+					   self->cache);
 		if (self->chain_hostnames) {
 			if (logmsg->flags & LF_LOCAL) {
 				/* local */
@@ -237,6 +248,7 @@ static int do_init_group(struct log_handler *c, struct syslog_config *cfg, struc
 	self->chain_hostnames = cfg->chain_hostnames;
 	self->use_dns = cfg->use_dns;
 	self->use_fqdn = cfg->use_fqdn;
+	self->cache = cfg->cache;
 	self->keep_hostname = cfg->keep_hostname;
 	res = 0;
 	for (drv = self->drivers; drv; drv = drv->next_driver) {
